'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var P = Promise;

/**
 * This function periodically invokes a synchronous function (a "truth test")
 * until it returns truthy value. The return value is a promise that settles
 * to these possible outcomes:
 *  - resolve if the truth test passes
 *  - reject if the truth test raises an Error
 *  - reject if the the operation times out
 * 
 * @async Promise
 * @param {Function} test - a function that returns a boolean
 * @param {Object} options
 * @param {Number} options.ttl - timeout in millis, default 10000
 * @param {Number} options.frequency - polling frequency in millis, default 100
 * @return void
 */
var until = function until(test) {
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var defaults = { ttl: 10000, frequency: 100 };
  var opts = _extends({}, defaults, options);

  return new P(function (resolve, reject) {
    var expire = function expire() {
      return reject(new Error('polling operation timed out'));
    };
    var timeout = setTimeout(expire, opts.ttl);

    var done = function done(e) {
      clearTimeout(timeout);
      e ? reject(e) : resolve();
    };

    var poll = function poll() {
      try {
        if (test()) {
          return done();
        }
      } catch (e) {
        return done(e);
      }

      setTimeout(poll, opts.frequency);
    };

    poll();
  });
};

exports.default = { until: until };
module.exports = exports['default'];