import lang from './lang';

/**
 * Convert a collection to a query string
 * @param {Object|Map|Array} query
 * @return {String}
 */
const stringify = (query) => {
  let entries = Array.isArray(query) ? query : lang.entries(query);

  let pair = (entry) => {
    let enc = encodeURIComponent;
    let [k, v] = entry;
    return `${enc(k)}=${enc(v)}`;
  };

  return entries.map(pair).join('&');
};


const parse = {};

parse.entries = (qs) => {
  // strip leading question mark
  if (qs && qs[0] === '?') {
    qs = qs.slice(1);
  }

  if (!qs.length) {
    return [];
  }

  let decode = qs => decodeURIComponent(qs).replace(/\+/g, ' ');

  let reducer = (out, pair) => {
    out.push(pair.split('=').slice(0, 2).map(decode));
    return out;
  };

  return qs.split('&').reduce(reducer, []);
};

parse.map = (qs) => {
  return new Map(parse.entries(qs));
};

parse.object = (qs) => {
  let reducer = (out, entry) => {
    let [k, v] = entry;
    out[k] = v;
    return out;
  };

  return lang.reduce(parse.entries(qs), reducer, {});
};

export default {
  stringify,
  parse,
};
