import {Tree} from './tree';
import {reduce, empty} from './lang';

/**
 * @param {String} translation
 * @param {Array} sub
 * @return {String}
 */
export class Translator extends Component {

  /**
   * Return configuration defaults
   * @param void
   * @return {Object}
   */
  defaults () {
    return {
      // Tree instance that contains translated messages
      translations: new Tree(),
      // suppress errors?
      suppress: false,
    };
  }

  constructor (options={}) {
    super(options);
    this.translations = this.option('translations', {pull: true});
  }

  /**
   * Tokenize the given expression on '.' and traverse the translation tree
   * until a match is found. If no match is found, return an empty string.
   * This is the main
   * @param {String} expr
   * @param {Object} subs
   * @return {String}
   */
  translate (expr, subs={}) {
    let {translations} = this;
    let {suppress} = this.options;

    try {
      return this.substitute(translations.path(expr), subs);
    }
    catch (e) {
      if (suppress) {
        return '';
      }
      throw e;
    }
  }

  /**
   * @param {String} translation
   * @param {Object} subs - string substitutions
   * @return {String}
   */
  substitute (translation, subs={}) {
    if (!translation || empty(subs)) {
      return translation;
    }

    let substitute = (translation, sub) => {
      let [key, value] = sub;
      let regex = new RegExp(`:${key}`, 'g');

      return translation.replace(regex, value);
    };

    return reduce(subs, substitute, translation);
  }

}


export default {
  Translator,
};
